@@ -16,6 +16,7 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA       *
  *************************************************************************************/
 
+#include <fcntl.h>
 #include <gcrypt.h>
 #include <stdio.h>
 #include <signal.h>
@@ -273,7 +274,7 @@ static int is_graphical_session(pam_handle_t *pamh)
 
 PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
-    pam_syslog(pamh, LOG_INFO, "%s: pam_sm_authenticate\n", logPrefix);
+    pam_syslog(pamh, LOG_DEBUG, "%s: pam_sm_authenticate\n", logPrefix);
     if (get_env(pamh, envVar) != NULL) {
         pam_syslog(pamh, LOG_INFO, "%s: we were already executed", logPrefix);
         return PAM_IGNORE;
@@ -419,7 +420,7 @@ static void execute_kwallet(pam_handle_t *pamh, struct passwd *userInfo, int toW
     fullSocket = NULL;
     unlink(local.sun_path);//Just in case it exists from a previous login
 
-    syslog(LOG_INFO, "%s: final socket path: %s", logPrefix, local.sun_path);
+    syslog(LOG_DEBUG, "%s: final socket path: %s", logPrefix, local.sun_path);
 
     size_t len = strlen(local.sun_path) + sizeof(local.sun_family);
     if (bind(envSocket, (struct sockaddr *)&local, len) == -1) {
@@ -468,7 +469,7 @@ static int better_write(int fd, const char *buffer, int len)
 {
     size_t writtenBytes = 0;
     while(writtenBytes < len) {
-        int result = write(fd, buffer + writtenBytes, len - writtenBytes);
+        ssize_t result = write(fd, buffer + writtenBytes, len - writtenBytes);
         if (result < 0) {
             if (errno != EAGAIN && errno != EINTR) {
                 return -1;
@@ -563,7 +564,7 @@ static void start_kwallet(pam_handle_t *pamh, struct passwd *userInfo, const cha
 
 PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
-    pam_syslog(pamh, LOG_INFO, "%s: pam_sm_open_session\n", logPrefix);
+    pam_syslog(pamh, LOG_DEBUG, "%s: pam_sm_open_session\n", logPrefix);
 
     if (get_env(pamh, envVar) != NULL) {
         pam_syslog(pamh, LOG_INFO, "%s: we were already executed", logPrefix);
@@ -616,20 +617,20 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, cons
 
 PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
-    pam_syslog(pamh, LOG_INFO, "%s: pam_sm_close_session", logPrefix);
+    pam_syslog(pamh, LOG_DEBUG, "%s: pam_sm_close_session", logPrefix);
     return PAM_SUCCESS;
 }
 
 PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
-    pam_syslog(pamh, LOG_INFO, "%s: pam_sm_setcred", logPrefix);
+    pam_syslog(pamh, LOG_DEBUG, "%s: pam_sm_setcred", logPrefix);
     return PAM_SUCCESS;
 }
 
 
 PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
-    pam_syslog(pamh, LOG_INFO, "%s: pam_sm_chauthtok", logPrefix);
+    pam_syslog(pamh, LOG_DEBUG, "%s: pam_sm_chauthtok", logPrefix);
     return PAM_SUCCESS;
 }
 
@@ -692,16 +693,21 @@ static void createNewSalt(pam_handle_t *pamh, const char *path, struct passwd *u
         free(dir);
 
         char *salt = gcry_random_bytes(KWALLET_PAM_SALTSIZE, GCRY_STRONG_RANDOM);
-        FILE *fd = fopen(path, "w");
+        const int fd = open(path, O_CREAT | O_WRONLY | O_TRUNC | O_CLOEXEC, 0600);
 
         //If the file can't be created
-        if (fd == NULL) {
+        if (fd == -1) {
             syslog(LOG_ERR, "%s: Couldn't open file: %s because: %d-%s", logPrefix, path, errno, strerror(errno));
             exit(-2);
         }
 
-        fwrite(salt, KWALLET_PAM_SALTSIZE, 1, fd);
-        fclose(fd);
+        const ssize_t wlen = write(fd, salt, KWALLET_PAM_SALTSIZE);
+        close(fd);
+        if (wlen != KWALLET_PAM_SALTSIZE) {
+            syslog(LOG_ERR, "%s: Short write to file: %s", logPrefix, path);
+            unlink(path);
+            exit(-2);
+        }
 
         exit(0); // success
     } else {
@@ -746,8 +752,8 @@ static int readSaltFile(pam_handle_t *pamh, char *path, struct passwd *userInfo,
             exit(-1);
         }
 
-        FILE *fd = fopen(path, "r");
-        if (fd == NULL) {
+        const int fd = open(path, O_RDONLY | O_CLOEXEC);
+        if (fd == -1) {
             syslog(LOG_ERR, "%s: Couldn't open file: %s because: %d-%s", logPrefix, path, errno, strerror(errno));
             free(path);
             close(readSaltPipe[1]);
@@ -755,8 +761,8 @@ static int readSaltFile(pam_handle_t *pamh, char *path, struct passwd *userInfo,
         }
         free(path);
         char salt[KWALLET_PAM_SALTSIZE] = {};
-        const int bytesRead = fread(salt, 1, KWALLET_PAM_SALTSIZE, fd);
-        fclose(fd);
+        const ssize_t bytesRead = read(fd, salt, KWALLET_PAM_SALTSIZE);
+        close(fd);
         if (bytesRead != KWALLET_PAM_SALTSIZE) {
             syslog(LOG_ERR, "%s: Couldn't read the full salt file contents from file. %d:%d", logPrefix, bytesRead, KWALLET_PAM_SALTSIZE);
             exit(-1);
